-------------------------------------------------------------------------------
--                 L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local daysInYear = 
{
  {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335},
  {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}
};

local holidayExtension = 
{
  ["HallowsEnd"] = (14*24*60)
};

local holidayList = {};

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to convert a date & time into a crude minute offset for determining
-- if a holiday is active
--
local function getHolidayTime(year, month, day, hour, min)
  if (year%4 == 0) then
    return (((((year*365)+daysInYear[1][month]+day)*24)+hour)*60)+min;
  else
    return (((((year*365)+daysInYear[2][month]+day)*24)+hour)*60)+min;
  end
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get a table of the active holidays
--
function NigglesHolidaysGetActive()
  -- Local Variables
  local currentDay;
  local currentMonth;
  local currentYear;
  local currentHour;
  local currentMin;
  local currentTime;
  local activeHolidays = {};

  -- Get the current date & time
  _, currentMonth, currentDay, currentYear = CalendarGetDate();
  currentHour, currentMin = GetGameTime();
  currentTime = getHolidayTime(currentYear, currentMonth, currentDay,
    currentHour, currentMin);

  -- Search for any active holidays
  for loop = 1, #holidayList do
    if ((currentTime >= holidayList[loop].start) and 
        (currentTime <= holidayList[loop].finish)) then
       tinsert(activeHolidays, holidayList[loop].event);
    end
  end

  return activeHolidays;
end

--
-- Function to initialise an array with holiday info for the previous, current
-- and next month
--
function NigglesHolidaysInitialise()
  -- Local Variables
  local numEvents;
  local eventCalType;
  local eventDate = {};
  local eventTexture;
  local eventType;
  local numDays;
  local eventSeq;
  local numHolidays = 0;
  local idx;

  -- Clear any existing holiday info
  wipe(holidayList);

  -- Get the current date
  _, eventDate.month, eventDate.day, eventDate.year = CalendarGetDate();

  -- Set the current month to properly initialise the Calendar functions
  CalendarSetAbsMonth(eventDate.month, eventDate.year);

  -- Create a list of holidays in the previous, current and next months
  for monthIdx = -1, 1 do
    _, _, numDays, _ = CalendarGetMonth(monthIdx);
    for dayIdx = 1, numDays do
      numEvents = CalendarGetNumDayEvents(monthIdx, dayIdx);
      for eventIdx = 1, numEvents do
        _, eventDate.hour, eventDate.min, eventCalType, eventSeq, eventType,
        eventTexture,  _, _ = CalendarGetDayEvent(monthIdx, dayIdx, eventIdx);
        if ((eventCalType == "HOLIDAY") and (eventType == 0)) then
          eventTexture = eventTexture:gsub("^Calendar_", "");
          if (eventSeq == "START") then
            -- Add the start event to the holiday info
            table.insert(holidayList,
              {
                start  = getHolidayTime(eventDate.year,
                  eventDate.month+monthIdx, dayIdx, eventDate.hour,
                  eventDate.min);
                finish = 0xFFFFFFFF,
                event  = eventTexture;
              });
          elseif (eventSeq == "END") then
            -- Search for a corresponding start event
            idx = 0;
            for loop = 1, #holidayList do
              if (eventTexture == holidayList[loop].event) then
                idx = loop;
                break;
              end
            end

            -- Add the end event to the holiday info
            if (idx == 0) then
              table.insert(holidayList,
                {
                  start = 0x00000000;
                  event = eventTexture;
                });
              idx = #holidayList;
            end
            holidayList[idx].finish = getHolidayTime(eventDate.year,
              eventDate.month+monthIdx, dayIdx, eventDate.hour, eventDate.min);
          end
        end
      end
    end
  end

  -- Add any extended holidays
  for _, holiday in ipairs(holidayList) do
    -- Check if an extended version of the holiday should be added
    if (holidayExtension[holiday.event] ~= nil) then
      -- Add an extended version of the holiday
      table.insert(holidayList,
      {
        start  = holiday.start,
        finish = holiday.finish+holidayExtension[holiday.event],
        event  = holiday.event.."Ex"
      });
    end
  end

  return;
end
