-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to initialise Merchant Search
--
function NigglesMerchantSearchInit()
  -- Local Variables
  local button;
  local texture;

  -- Create the textures used to highlight the buttons
  for buttonIdx = 1, MERCHANT_ITEMS_PER_PAGE do
    button = _G["MerchantItem"..buttonIdx.."ItemButton"];
    texture = button:CreateTexture(button:GetName().."Glow", "BACKGROUND");
    texture:SetTexture("Interface\\SpellActivationOverlay\\IconAlert");
    texture:SetTexCoord(0.00781250, 0.50781250, 0.27734375, 0.52734375);
    texture:ClearAllPoints();
    texture:SetPoint("TOPLEFT", -18, 18);
    texture:SetPoint("BOTTOMRIGHT", 18, -18);
  end

  -- Add a hook to the function used to update the Merchant frame
  hooksecurefunc("MerchantFrame_Update", NigglesMerchantSearchUpdate);

  return;
end

--
-- Function to enable/disable Merchant Search
--
function NigglesMerchantSearchEnable()
  -- Check if Merchant Search is enabled
  if (NigglesDBC.settings.UiMerchantSearch) then
    -- Show the search text edit box
    _G["NigglesMerchantSearchBox"]:Show();

    -- Update the Merchant's frame, if shown
    if (_G["MerchantFrame"]:IsShown()) then
      NigglesMerchantSearch()
    end
  else
    -- Hide the search text edit box
    _G["NigglesMerchantSearchBox"]:Hide();

    -- Hide all the textures
    for buttonIdx = 1, MERCHANT_ITEMS_PER_PAGE do
      _G["MerchantItem"..buttonIdx.."ItemButtonGlow"]:Hide();
    end
  end

  return;
end

--
-- Function used to update the Merchant frame, based on the search text
--
function NigglesMerchantSearchUpdate()
  -- Local Variables
  local buttonText;
  local searchText = _G["NigglesMerchantSearchBox"]:GetText();

  -- Check if searching is enabled
  if (NigglesDBC.settings.UiMerchantSearch) then
    -- Check if there is any search text to match against
    if ((MerchantFrame.selectedTab == 1) and
        (searchText ~= SEARCH) and
        (searchText ~= "") and
        (searchText ~= nil)) then
      -- Convert the search text to lowercase
      searchText = string.lower(searchText);

      -- Show the textures for those buttons that match the search text
      for buttonIdx = 1, MERCHANT_ITEMS_PER_PAGE do
        buttonText = _G["MerchantItem"..buttonIdx.."Name"]:GetText();
        if ((_G["MerchantItem"..buttonIdx.."ItemButton"]:IsShown()) and
            (buttonText ~= nil) and
            (string.find(string.lower(buttonText), searchText, 1, true) ~=
              nil)) then
          _G["MerchantItem"..buttonIdx.."ItemButtonGlow"]:Show();
        else
          _G["MerchantItem"..buttonIdx.."ItemButtonGlow"]:Hide();
        end
      end
    else
      -- Hide all the textures
      for buttonIdx = 1, MERCHANT_ITEMS_PER_PAGE do
        _G["MerchantItem"..buttonIdx.."ItemButtonGlow"]:Hide();
      end
    end
  end

  return;
end
