-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local stateColour =
{
  {r = 0.0, g = 1.0, b = 0.0},
  {r = 1.0, g = 0.0, b = 0.0},
  {r = 0.4, g = 0.4, b = 0.5}
};

local environmentInfo =
{
  {name = "run",  flag = 0x01, title = "MountRun" },
  {name = "fly",  flag = 0x02, title = "MountFly" },
  {name = "swim", flag = 0x08, title = "MountSwim"}
};

local TOGGLE_STATE_SELECTED   = 1;
local TOGGLE_STATE_UNSELECTED = 2;
local TOGGLE_STATE_DISABLED   = 3;

-------------------------------------------------------------------------------
--                   S  L  A  S  H     C  O  M  M  A  N  D  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function for processing clicks on a companion toggle button
--
function NigglesCompanionToggleClicked(toggle)
  -- Local Variables
  local button;

  -- Determine the type of companion displayed by the companion page
  if (SpellBookCompanionsFrame.mode == "CRITTER") then
    if (NigglesDBC.critters[toggle.spellId] ~= nil) then
      NigglesDBC.critters[toggle.spellId] = nil;
      NigglesCompanionToggleSetState(toggle, TOGGLE_STATE_UNSELECTED);
    else
      NigglesDBC.critters[toggle.spellId] = 1;
      NigglesCompanionToggleSetState(toggle, TOGGLE_STATE_SELECTED);
    end
  else
    -- Toggle the display of the Mount flyout
    if ((NigglesMountFlyout:IsShown()) and
        (NigglesMountFlyout.toggle == toggle)) then
      -- Hide the flyout
      NigglesMountFlyout:Hide();

      -- Set all toggles to the left
      for idx = 1, NUM_COMPANIONS_PER_PAGE do
        _G["NigglesCompanionToggle"..idx].outline:SetTexCoord(
          0.0, 0.40625, 0.0, 0.515625);
      end
    else
      -- Set the toggle to point to the right
      toggle.outline:SetTexCoord(0.40625, 0.0, 0.0, 0.515625);

      -- Set all other toggles to the left
      for idx = 1, NUM_COMPANIONS_PER_PAGE do
        if (_G["NigglesCompanionToggle"..idx] ~= toggle) then
          _G["NigglesCompanionToggle"..idx].outline:SetTexCoord(
            0.0, 0.40625, 0.0, 0.515625);
        end
      end

      -- Position the flyout
      NigglesMountFlyout.toggle = toggle;
      NigglesMountFlyout:ClearAllPoints();
      NigglesMountFlyout:SetPoint("CENTER", toggle, "CENTER", -31, 2);
      NigglesMountFlyout:SetFrameLevel(toggle:GetFrameLevel()+1);

      -- Set the state of the flyout's buttons
      for idx = 1, #environmentInfo do
        button = NigglesMountFlyout[environmentInfo[idx].name];
        if (bit.band(toggle.mountFlags, environmentInfo[idx].flag) ~= 0) then
          button:Enable();
          button.icon:SetVertexColor(1.0, 1.0, 1.0);
          if ((NigglesDBC.mounts[toggle.spellId] ~= nil) and
              (bit.band(NigglesDBC.mounts[toggle.spellId],
                environmentInfo[idx].flag) ~= 0)) then
            button:SetChecked(true);
          else
            button:SetChecked(false);
          end
        else
          button:Disable();
          button.icon:SetVertexColor(0.4, 0.4, 0.4);
          button:SetChecked(false);
        end
      end

      -- Show the flyout
      NigglesMountFlyout:Show();
    end
  end

  return;
end

--
-- Function to change the colours of the 3 segments of a companion toggle
--
function NigglesCompanionToggleSetState(toggle, run, fly, swim)
  -- Make sure the fly and swim segments have a state
  fly  = (fly  or run);
  swim = (swim or run);
    
  -- Set the colour of each segment
  if ((run  ~= nil) and (run  >= 1) and (run  <= #stateColour)) then
    toggle.run:SetTexture(stateColour[run].r, stateColour[run].g,
      stateColour[run].b);
  end
  if ((fly  ~= nil) and (fly  >= 1) and (fly  <= #stateColour)) then 
    toggle.fly:SetTexture(stateColour[fly].r, stateColour[fly].g,
      stateColour[fly].b);
  end
  if ((swim ~= nil) and (swim >= 1) and (swim <= #stateColour)) then
    toggle.swim:SetTexture(stateColour[swim].r, stateColour[swim].g,
      stateColour[swim].b);
  end

  return;
end

--
-- Function to process clicks on a button in the Niggles Mount Flyout
--
function NigglesMountFlyoutButtonClicked(button)
  -- Local Variables
  local environmentIdx = button:GetID();
  local toggle         = button:GetParent().toggle;

  -- Update the database with the mount's new state
  NigglesDBC.mounts[toggle.spellId] = 
    bit.bxor((NigglesDBC.mounts[toggle.spellId] or 0),
      environmentInfo[environmentIdx].flag);

  -- Update the state of the mount's toggle
  NigglesMountToggleSetState(toggle);

  return;
end

--
-- Function to display the tooltip for a Mount Flyout button
--
function NigglesMountFlyoutButtonOnEnter(button)
  -- Local Variables
  local buttonId = button:GetID();

  -- Check the button's ID is valid
  if ((buttonId ~= nil) and
      (buttonId >= 1) and
      (buttonId <= #environmentInfo)) then
    -- Set the game tooltip
    GameTooltip:SetOwner(button, "ANCHOR_RIGHT", 6, -button:GetHeight()-2);
    GameTooltip:SetText(L[environmentInfo[buttonId].title],
      HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
    GameTooltip:AddLine(L[environmentInfo[buttonId].title.."Tooltip"],
      NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true);
    GameTooltip:Show();
  else
    -- Hide the game tooltip
    GameTooltip:Hide();
  end

  return;
end

--
-- Function to change the colours of the 3 segments of a companion toggle based
-- on the possible states for the toggle's mount
--
function NigglesMountToggleSetState(toggle)
  -- Local Variables
  local state = {};

  -- Work out the state for each environment
  for idx = 1, #environmentInfo do
    if (bit.band(toggle.mountFlags, environmentInfo[idx].flag) ~= 0) then
      if ((NigglesDBC.mounts[toggle.spellId] ~= nil) and
          (bit.band(NigglesDBC.mounts[toggle.spellId],
            environmentInfo[idx].flag) ~= 0)) then
        state[idx] = TOGGLE_STATE_SELECTED;
      else
        state[idx] = TOGGLE_STATE_UNSELECTED;
      end
    else
      state[idx] = TOGGLE_STATE_DISABLED;
    end
  end

  -- Update the toggle's state
  NigglesCompanionToggleSetState(toggle, state[1], state[2], state[3]);

  return;
end

--
-- Function to update the state of all the toggles on the spell book's
-- companion page.
--
function NigglesSpellBookCompanionUpdate()
  -- Local Variables
  local toggle;
  local offset;

  -- Hide the Mount flyout
  NigglesMountFlyout:Hide();
  NigglesMountFlyout.toggle = nil;

  -- Find the companion offset for the current page
  offset = SpellBook_GetCurrentPage();
  if (offset ~= nil) then
    offset = offset-1;
  end
  offset = (offset or 0)*NUM_COMPANIONS_PER_PAGE;
    
  -- Update the toggle for each button on the page
  for idx = 1, NUM_COMPANIONS_PER_PAGE do
    -- Assign some info to more convenient variables
    toggle         = _G["NigglesCompanionToggle"..idx];
    toggle.spellId = _G["SpellBookCompanionButton"..idx].spellID;

    -- Update the state of the current button
    if ((SpellBookCompanionsFrame.mode == "CRITTER") and
        (NigglesDBC.settings.CrittersEnabled == true) and
        (toggle.spellId ~= nil)) then
      -- Set the outline for the toggle
      toggle.outline:SetTexCoord(0.5, 0.90625, 0.0, 0.515625);

      -- Work out what colour to display in the toggle
      if (NigglesDBC.critters[toggle.spellId] == 1) then
        NigglesCompanionToggleSetState(toggle, TOGGLE_STATE_SELECTED);
      else
        NigglesCompanionToggleSetState(toggle, TOGGLE_STATE_UNSELECTED);
      end

      -- Make sure the toggle is shown
      toggle:Show();
    elseif ((SpellBookCompanionsFrame.mode == "MOUNT") and
            (NigglesDBC.settings.MountsEnabled == true) and
            (toggle.spellId ~= nil)) then
      -- Set the outline for the toggle
      toggle.outline:SetTexCoord(0, 0.40625, 0.0, 0.515625);

      -- Get the mount flags for the companion
      _, _, _, _, _, toggle.mountFlags =
        GetCompanionInfo("MOUNT", offset+idx);

      -- Work out what colour to display in each segment of the toggle
      NigglesMountToggleSetState(toggle);

      -- Make sure the toggle is shown
      toggle:Show();
    else
      -- Hide the toggle
      toggle:Hide();
    end
  end

  return;
end

--
-- Function to create toggles for each button on the companion page of the
-- spell book and hook into the function used to update the page.
--
function NigglesSpellBookInit()
  -- Create a toggle for each button on the companion page
  for idx = 1, NUM_COMPANIONS_PER_PAGE do
    CreateFrame("Button", "NigglesCompanionToggle"..idx,
      _G["SpellBookCompanionButton"..idx], "NigglesCompanionToggleTemplate");
  end

  -- Hook into the updating of the companion page
  hooksecurefunc("SpellBook_UpdateCompanionsFrame", 
    NigglesSpellBookCompanionUpdate);

  return;
end
